﻿Imports Microsoft.Office.Interop
Imports System.IO
Imports System.Text

Public Class ExcelClass

    '画面パラメータの構造体
    Public Structure FormParameter

        Public pTargetXlsFile As String
        Public pJpegQuality As Integer
        Public pZeroSuppress As Integer
        Public pHtmlImgAlt As String
        Public pHtmlImgSrc As String
        Public pTrimChecked As Boolean
        Public pWorkSheetInfo() As WorkSheetInfo    'ワークシート情報リスト
        Public pImageAllCount As Integer            '処理対象全画像数

    End Structure

    'ワークブック情報の構造体
    Public Structure WorkSheetInfo

        Public OutputEnable As Boolean  '出力有無
        Public SheetName As String      'ワークシートト名称
        Public ImageCount As Integer    'シート内画像数

    End Structure

    'Excelファイルの情報を取得
    '引数：Excelファイルのパス,ワークブック情報の構造体配列(参照渡し)
    Public Sub GetFileInfo(ByVal TargetFile As String, ByRef WorkSheetInfo() As WorkSheetInfo)

        'Excel関連変数  
        Dim xlApp As Excel.Application
        Dim xlBooks As Excel.Workbooks
        Dim xlBook As Excel.Workbook
        Dim xlSheets As Excel.Sheets
        Dim xlSheet As Excel.Worksheet
        Dim xlShapes As Excel.Shapes

        Try

            'Excel起動
            xlApp = New Excel.Application

            'ファイルオープン&ワークシート取得
            xlBooks = xlApp.Workbooks
            xlBook = xlBooks.Open(TargetFile)
            xlSheets = xlBook.Worksheets

            'ブック内シート読み込み
            For I As Integer = 1 To xlSheets.Count

                ReDim Preserve WorkSheetInfo(I)     'ワークシート情報配列確保

                Try
                    xlSheet = DirectCast(xlSheets(I), Excel.Worksheet)
                    xlShapes = xlSheet.Shapes

                    WorkSheetInfo(I).OutputEnable = True                  'とりあえず許可    
                    WorkSheetInfo(I).SheetName = xlSheet.Name             'シート名
                    WorkSheetInfo(I).ImageCount = xlShapes.Count          '画像数

                Finally

                    'Excel変数解放
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(xlShapes)
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(xlSheet)

                End Try

            Next

            xlBook.Close(False)             'ブック閉じる
            xlApp.Quit()                    'Excel終了

        Finally

            'Excel変数解放
            System.Runtime.InteropServices.Marshal.ReleaseComObject(xlSheets)
            System.Runtime.InteropServices.Marshal.ReleaseComObject(xlBooks)
            System.Runtime.InteropServices.Marshal.ReleaseComObject(xlApp)

        End Try

        ''ExcelのCOMオブジェクト解放確認
        'Dim st As Integer = System.Environment.TickCount
        'Do While System.Environment.TickCount - st < 5000
        '    Application.DoEvents()
        '    System.Threading.Thread.Sleep(500)
        '    If Process.GetProcessesByName("Excel").Length = 0 Then
        '        MessageBox.Show("Excel.EXE は解放されました。")
        '        Exit Do
        '    End If
        'Loop
        'If Process.GetProcessesByName("Excel").Length >= 1 Then
        '    MessageBox.Show("まだ Excel.EXE が起動しています。")
        '    '一度メッセージボックスを表示すると解放されるようなので再度確認
        '    If Process.GetProcessesByName("Excel").Length = 0 Then
        '        MessageBox.Show("Excel.EXE は解放されました。")
        '    End If
        'End If

    End Sub

    'ワークブック上の画像(シェイプ)をJPEGに保存
    '引数：画面パラメータの構造体,プログレスバーの参照
    Public Sub PictureSave(ByVal FormParameter As FormParameter, ByRef pProgressBar As ProgressBar)

        'Excel関連変数  
        Dim xlApp As Excel.Application
        Dim xlBooks As Excel.Workbooks
        Dim xlBook As Excel.Workbook
        Dim xlSheets As Excel.Sheets
        Dim xlSheet As Excel.Worksheet
        Dim xlShapes As Excel.Shapes
        Dim xlShape As Excel.Shape
        Dim xlRange As Excel.Range
        Dim xlInterior As Excel.Interior

        Dim TargetBitmap As Bitmap                      'イメージ格納変数
        Dim clsImage As ImageClass                      'イメージ処理クラス
        Dim OutputPath As String                        '出力先フォルダ
        Dim OutputFileName As String                    '出力ファイル名
        Dim OutputFileCount As Integer                  'ファイル名連番カウント
        Dim ZeroStringLength As Integer                 'ゼロ前詰め桁数(処理画像数から算出)

        Dim HTMLFileName As String
        Dim sw As IO.StreamWriter

        'Excelファイルの直下に出力用フォルダ作成
        OutputPath = Path.GetDirectoryName(FormParameter.pTargetXlsFile) + "\画像保存" + Format(Now, "yyyyMMddmmss")
        Directory.CreateDirectory(OutputPath)

        'プログレスバー設定
        pProgressBar.Minimum = 0
        pProgressBar.Maximum = FormParameter.pImageAllCount
        pProgressBar.Value = 0

        Try

            'Excel起動
            xlApp = New Excel.Application

            'ファイルオープン&ワークシート取得
            xlBooks = xlApp.Workbooks
            xlBook = xlBooks.Open(FormParameter.pTargetXlsFile)
            xlSheets = xlBook.Worksheets

            '画像処理クラスのインスタンス
            clsImage = New ImageClass

            'HTMLタグ出力ファイルの設定とオープン
            HTMLFileName = OutputPath + "\Html.txt"
            sw = New IO.StreamWriter(HTMLFileName, True, Encoding.GetEncoding("Shift_JIS"))

            'ゼロ前詰め桁数(処理画像数から算出)
            ZeroStringLength = FormParameter.pZeroSuppress

            'ブック内シート読み込み
            For I As Integer = 1 To xlSheets.Count

                'シートの出力判断
                If FormParameter.pWorkSheetInfo(I).OutputEnable = True Then

                    '画像処理カウントのリセット
                    OutputFileCount = 0

                    Try
                        xlSheet = DirectCast(xlSheets(I), Excel.Worksheet)
                        xlShapes = xlSheet.Shapes

                        'シート内シェイプ検索
                        For Each xlShape In xlShapes

                            Try
                                'クリップボードにコピー(ビットマップ形式で)
                                xlShape.CopyPicture(Appearance:=Excel.XlPictureAppearance.xlScreen, Format:=Excel.XlCopyPictureFormat.xlBitmap)

                            Finally

                                'Excel変数解放
                                System.Runtime.InteropServices.Marshal.ReleaseComObject(xlShape)

                            End Try

                            'クリップボードからビットマップ変数へ
                            TargetBitmap = Clipboard.GetImage()

                            'ファイル名カウント
                            OutputFileCount = OutputFileCount + 1

                            'ファイル名生成(シート名+連番)
                            OutputFileName = xlSheet.Name + OutputFileCount.ToString(New String("0", ZeroStringLength)) + ".jpg"

                            'トリミングするか否か
                            If FormParameter.pTrimChecked = True Then

                                'トリミング(参照でビットマップ渡す)   
                                clsImage.ImageTrim(TargetBitmap)

                            End If

                            'ビットマップ変数をJPEG保存
                            clsImage.JpegImageSave(TargetBitmap, OutputPath + "\" + OutputFileName, FormParameter.pJpegQuality)

                            'HTML出力
                            Call OutputHTML(sw, TargetBitmap, OutputFileName, FormParameter.pHtmlImgSrc, OutputFileCount.ToString(New String("0", ZeroStringLength)), FormParameter.pHtmlImgAlt)

                            'プログレスバー進行
                            pProgressBar.Increment(1)

                        Next

                    Finally

                        'Excel変数解放
                        System.Runtime.InteropServices.Marshal.ReleaseComObject(xlShapes)
                        System.Runtime.InteropServices.Marshal.ReleaseComObject(xlSheet)

                    End Try

                End If

            Next

            'HTML出力ファイルクローズ    
            sw.Close()

            xlBook.Close(False)             'ブック閉じる
            xlApp.Quit()                    'Excel終了

        Finally

            If Not sw Is Nothing Then
                sw.Dispose()
            End If

            clsImage = Nothing

            'Excel変数解放
            System.Runtime.InteropServices.Marshal.ReleaseComObject(xlSheets)
            System.Runtime.InteropServices.Marshal.ReleaseComObject(xlBooks)
            System.Runtime.InteropServices.Marshal.ReleaseComObject(xlApp)

        End Try

        ''ExcelのCOMオブジェクト解放確認
        'Dim st As Integer = System.Environment.TickCount
        'Do While System.Environment.TickCount - st < 5000
        '    Application.DoEvents()
        '    System.Threading.Thread.Sleep(500)
        '    If Process.GetProcessesByName("Excel").Length = 0 Then
        '        MessageBox.Show("Excel.EXE は解放されました。")
        '        Exit Do
        '    End If
        'Loop
        'If Process.GetProcessesByName("Excel").Length >= 1 Then
        '    MessageBox.Show("まだ Excel.EXE が起動しています。")
        '    '一度メッセージボックスを表示すると解放されるようなので再度確認
        '    If Process.GetProcessesByName("Excel").Length = 0 Then
        '        MessageBox.Show("Excel.EXE は解放されました。")
        '    End If
        'End If

    End Sub

    '画像情報をimgタグに出力
    '引数：出力ファイル(参照),ビットマップ,ファイル名,src属性パス部分,連番文字列,alt属性
    Private Sub OutputHTML(ByRef sw As IO.StreamWriter, ByVal TargetBitmap As Bitmap, ByVal FileName As String, ByVal Src As String, ByVal FileCount As String, ByVal Alt As String)

        Dim HTMLString As String

        'HTML文字列組み立て
        HTMLString = ""
        HTMLString = HTMLString + "<img src="""
        HTMLString = HTMLString + Src
        HTMLString = HTMLString + FileName
        HTMLString = HTMLString + """ alt="""
        HTMLString = HTMLString + Alt + FileCount
        HTMLString = HTMLString + """ height="""
        HTMLString = HTMLString + TargetBitmap.Height.ToString
        HTMLString = HTMLString + """ width="""
        HTMLString = HTMLString + TargetBitmap.Width.ToString
        HTMLString = HTMLString + """ />"

        'ファイルに出力
        sw.Write(HTMLString)
        sw.Write(vbCrLf)

    End Sub

End Class
