﻿Public Class Form1

    'フォームロード時
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        '設定ファイルから画面項目を読み込む
        Call ReadConfig()

        'リスト表示初期化
        Me.lstWorkSheetInfo.Columns.Add("シート名", 210, HorizontalAlignment.Left)
        Me.lstWorkSheetInfo.Columns.Add("画像数", 50, HorizontalAlignment.Right)
        Me.lstWorkSheetInfo.CheckBoxes = True
        Me.lstWorkSheetInfo.Items.Clear()

        'プログレスバー初期化
        Me.ProgressBar1.Value = 0

        '実行ボタンの制御
        Call ExecEnable()

    End Sub

    'ファイル選択ボタン
    Private Sub btnFileSelect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnFileSelect.Click

        Dim TargetXlsFileName As String
        Dim WorkSheetInfo(0) As ExcelClass.WorkSheetInfo 'ワークシート情報取得
        Dim clsExcel As ExcelClass

        'ファイルダイアログ関数呼び出しファイルパス取得
        TargetXlsFileName = FileSelect()

        '画面に表示
        Me.txtTargetXlsFile.Text = TargetXlsFileName

        '実行ボタンの制御
        Call ExecEnable()

        'ファイル名がある場合は実行
        If TargetXlsFileName <> "" Then

            Try

                'Excelクラスインスタンス作成
                clsExcel = New ExcelClass()

                'ファイルの情報取得(結果がWorkSheetInfoに格納される)
                clsExcel.GetFileInfo(TargetXlsFileName, WorkSheetInfo)

            Finally

                clsExcel = Nothing

            End Try

        End If

        'リストに表示
        Call DispList(WorkSheetInfo)

    End Sub

    '実行ボタン
    Private Sub btnExec_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnExec.Click

        Dim FormParameter As ExcelClass.FormParameter       '画面情報パラメータ
        Dim result As DialogResult
        Dim clsExcel As ExcelClass


        '各入力値チェック
        If Me.txtTargetXlsFile.Text = "" Then
            MessageBox.Show("Excelファイルが選択されていません。")
            Exit Sub
        End If
        If IsNumeric(Me.txtJpegQuality.Text) = False Then
            MessageBox.Show("Jpeg品質は数値を入力してください。")
            Exit Sub
        End If
        If Me.txtJpegQuality.Text < 25 Or Me.txtJpegQuality.Text > 100 Then
            MessageBox.Show("Jpeg品質は25～100の範囲で入力してください。")
            Exit Sub
        End If
        If IsNumeric(Me.txtZeroSuppress.Text) = False Then
            MessageBox.Show("0前詰め桁数は数値を入力してください。")
            Exit Sub
        End If

        '画面情報を構造体に設定
        FormParameter.pTargetXlsFile = Me.txtTargetXlsFile.Text
        FormParameter.pJpegQuality = Me.txtJpegQuality.Text
        FormParameter.pZeroSuppress = Me.txtZeroSuppress.Text
        FormParameter.pHtmlImgAlt = Me.txtHtmlImgAlt.Text
        FormParameter.pHtmlImgSrc = Me.txtHtmlImgSrc.Text
        FormParameter.pTrimChecked = Me.chkTrim.Checked
        FormParameter.pImageAllCount = 0
        For I As Integer = 1 To lstWorkSheetInfo.Items.Count

            ReDim Preserve FormParameter.pWorkSheetInfo(I)     'ワークシート情報配列確保
            FormParameter.pWorkSheetInfo(I).OutputEnable = lstWorkSheetInfo.Items(I - 1).Checked
            FormParameter.pWorkSheetInfo(I).SheetName = lstWorkSheetInfo.Items(I - 1).Text
            FormParameter.pWorkSheetInfo(I).ImageCount = lstWorkSheetInfo.Items(I - 1).SubItems(1).Text
            'トータルの画像数(チェック行のみ)
            If FormParameter.pWorkSheetInfo(I).OutputEnable = True Then
                FormParameter.pImageAllCount = FormParameter.pImageAllCount + FormParameter.pWorkSheetInfo(I).ImageCount
            End If

        Next

        '画像数が0の場合は処理しない
        If FormParameter.pImageAllCount = 0 Then
            MessageBox.Show("処理対象画像がありません(チェックが付いていない等)。")
            Exit Sub
        End If

        '実行確認
        result = MessageBox.Show("画像保存を実行しますか？", "確認", MessageBoxButtons.YesNo)
        If result = Windows.Forms.DialogResult.Yes Then

            Try
                'Excelクラスインスタンス作成
                clsExcel = New ExcelClass()

                '画像保存処理
                clsExcel.PictureSave(FormParameter, Me.ProgressBar1)

            Finally

                clsExcel = Nothing

            End Try

            '完了メッセージ
            MessageBox.Show("画像の保存が完了しました。")
            Me.ProgressBar1.Value = 0

        End If

    End Sub

    'リストにワークシート情報表示
    'ワークシート情報の配列
    Private Sub DispList(ByVal WorkSheetInfo() As ExcelClass.WorkSheetInfo)

        Dim ListItem As ListViewItem

        If WorkSheetInfo.Length > 1 Then

            For I As Integer = 1 To WorkSheetInfo.Length - 1

                ListItem = New ListViewItem()

                'アイテムの作成
                ListItem.Checked = True
                ListItem.Text = WorkSheetInfo(I).SheetName
                ListItem.SubItems.Add(WorkSheetInfo(I).ImageCount.ToString) 'サブアイテムの追加

                'リストに追加
                Me.lstWorkSheetInfo.Items.Add(ListItem)

            Next

        Else

            Me.lstWorkSheetInfo.Items.Clear()

        End If

    End Sub

    'ファイルダイアログ関数
    '引数：なし
    '戻り値：ファイルのフルパス
    Private Function FileSelect() As String

        'ファイルダイアログのインスタンス
        Dim FileDialog As New OpenFileDialog()

        'タイトル設定
        FileDialog.Title = "Excel画像保存ツール"

        'ファイルフィルタ設定
        FileDialog.Filter = "Excelワークブック|*.xls;*.xlsx"

        ' ダイアログを表示し、戻り値が [OK] の場合は、選択したファイルを表示する
        If FileDialog.ShowDialog() = DialogResult.OK Then
            FileSelect = FileDialog.FileName
        Else
            FileSelect = ""
        End If

        'オブジェクト破棄
        FileDialog.Dispose()

    End Function

    '実行ボタンの許可制御
    Private Sub ExecEnable()

        'ファイル名の有無で判断
        If Me.txtTargetXlsFile.Text <> "" Then
            Me.btnExec.Enabled = True
        Else
            Me.btnExec.Enabled = False
        End If

    End Sub

    '設定ファイル→画面項目読み込み
    Private Sub ReadConfig()

        'コントロールに値を設定する際にイベントが発生しないようにする
        RemoveHandler txtJpegQuality.TextChanged, AddressOf txtJpegQuality_TextChanged
        RemoveHandler txtZeroSuppress.TextChanged, AddressOf txtZeroSuppress_TextChanged
        RemoveHandler txtHtmlImgAlt.TextChanged, AddressOf txtHtmlImgAlt_TextChanged
        RemoveHandler txtHtmlImgSrc.TextChanged, AddressOf txtHtmlImgSrc_TextChanged
        RemoveHandler chkTrim.CheckedChanged, AddressOf chkTrim_CheckedChanged

        Me.txtJpegQuality.Text = My.Settings.JpegQuality
        Me.txtZeroSuppress.Text = My.Settings.ZeroSuppress
        Me.txtHtmlImgAlt.Text = My.Settings.HtmlImgAlt
        Me.txtHtmlImgSrc.Text = My.Settings.HtmlImgSrc
        Me.chkTrim.Checked = My.Settings.chkTrim

        'イベント再設定
        AddHandler txtJpegQuality.TextChanged, AddressOf txtJpegQuality_TextChanged
        AddHandler txtZeroSuppress.TextChanged, AddressOf txtZeroSuppress_TextChanged
        AddHandler txtHtmlImgAlt.TextChanged, AddressOf txtHtmlImgAlt_TextChanged
        AddHandler txtHtmlImgSrc.TextChanged, AddressOf txtHtmlImgSrc_TextChanged
        AddHandler chkTrim.CheckedChanged, AddressOf chkTrim_CheckedChanged

    End Sub

    '画面項目→設定ファイル書き込み
    Private Sub WriteConfig()

        My.Settings.JpegQuality = Me.txtJpegQuality.Text
        My.Settings.ZeroSuppress = Me.txtZeroSuppress.Text
        My.Settings.HtmlImgAlt = Me.txtHtmlImgAlt.Text
        My.Settings.HtmlImgSrc = Me.txtHtmlImgSrc.Text
        My.Settings.chkTrim = Me.chkTrim.Checked

    End Sub

    '以下各項目変更時に設定を保存
    Private Sub txtTargetXlsFile_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtTargetXlsFile.TextChanged
        Call WriteConfig()
    End Sub

    Private Sub txtJpegQuality_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtJpegQuality.TextChanged
        Call WriteConfig()
    End Sub

    Private Sub txtZeroSuppress_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtZeroSuppress.TextChanged
        Call WriteConfig()
    End Sub

    Private Sub txtHtmlImgAlt_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtHtmlImgAlt.TextChanged
        Call WriteConfig()
    End Sub

    Private Sub txtHtmlImgSrc_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtHtmlImgSrc.TextChanged
        Call WriteConfig()
    End Sub

    Private Sub chkTrim_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkTrim.CheckedChanged
        Call WriteConfig()
    End Sub

End Class
