﻿Imports System
Imports System.Drawing
Imports System.Drawing.Imaging

Public Class ImageClass

    'Jpeg画像保存
    '引数：画像オブジェクト,保存ファイル名(フルパス),JPEG品質値
    Public Sub JpegImageSave(ByVal TargetBitmap As Bitmap, ByVal SaveFilePath As String, ByVal JpegQuality As Integer)

        Dim quality As Int64 = JpegQuality
        Dim jpgEncoder As ImageCodecInfo = Nothing

        ' JPEG用のエンコーダの取得
        For Each ici As ImageCodecInfo In ImageCodecInfo.GetImageEncoders()
            If ici.FormatID = ImageFormat.Jpeg.Guid Then
                jpgEncoder = ici
                Exit For
            End If
        Next

        ' エンコーダに渡すパラメータの作成
        Dim encParam As New EncoderParameter(Encoder.Quality, quality)

        ' パラメータを配列に格納
        Dim encParams As New EncoderParameters(1)
        encParams.Param(0) = encParam

        ' 画像の保存
        TargetBitmap.Save(SaveFilePath, jpgEncoder, encParams)

        encParam.Dispose()
        encParams.Dispose()

    End Sub

    'トリミング
    '引数：画像オブジェクト(参照渡し)
    Public Sub ImageTrim(ByRef TargetBitmap As Bitmap)

        '上下左右の変数
        Dim TrimTop As Integer
        Dim TrimBottom As Integer
        Dim TrimLeft As Integer
        Dim TrimRight As Integer

        '上の余白量取得
        TrimTop = CalcMargin(TargetBitmap, 1)

        '上部の余白量とTargetBitmap.Heightが同じなら処理終了(背景と同色の画像とみなす)
        If TrimTop = TargetBitmap.Height Then
            Exit Sub
        End If

        '下左右の余白量取得
        TrimBottom = CalcMargin(TargetBitmap, 2)
        TrimLeft = CalcMargin(TargetBitmap, 3)
        TrimRight = CalcMargin(TargetBitmap, 4)

        ' 画像を切り抜く
        Dim rect As New Rectangle(TrimLeft, TrimTop, TargetBitmap.Width - TrimLeft - TrimRight, TargetBitmap.Height - TrimTop - TrimBottom)

        'トリミング結果を画像オブジェクトに返す
        TargetBitmap = TargetBitmap.Clone(rect, TargetBitmap.PixelFormat)

    End Sub

    '余白量算出
    '引数：画像オブジェクト(参照渡し),余白算出位置(1=上 2=下 3=左 4=右)
    '戻り値：余白量(ピクセル)
    Private Function CalcMargin(ByRef TargetBitmap As Bitmap, ByVal Mode As Integer) As Integer

        Dim OutLoopStart As Integer
        Dim OutLoopEnd As Integer
        Dim OutLoopStep As Integer
        Dim InLoopStart As Integer
        Dim InLoopEnd As Integer
        Dim InLoopStep As Integer

        Dim PixelColor As Color       '画像のカラー構造体
        Dim PixelHexStr As String     '画像ピクセルの16進数文字列
        Dim MarginHexStr As String    '余白の16進数文字列

        Dim MarginPixel As String     '余白量(ピクセル)

        '余白のカラーコードを設定("ffffff"固定でよい)
        MarginHexStr = "ffffff"

        'モードにより初期パラメータを変える
        Select Case Mode
            Case 1          '上
                OutLoopStart = 0
                OutLoopEnd = TargetBitmap.Height - 1
                OutLoopStep = 1
                InLoopEnd = TargetBitmap.Width - 1
            Case 2          '下
                OutLoopStart = TargetBitmap.Height - 1
                OutLoopEnd = 0
                OutLoopStep = -1
                InLoopEnd = TargetBitmap.Width - 1
            Case 3          '左 
                OutLoopStart = 0
                OutLoopEnd = TargetBitmap.Width - 1
                OutLoopStep = 1
                InLoopEnd = TargetBitmap.Height - 1
            Case 4          '右
                OutLoopStart = TargetBitmap.Width - 1
                OutLoopEnd = 0
                OutLoopStep = -1
                InLoopEnd = TargetBitmap.Height - 1
        End Select

        '以下は数値固定でよい
        InLoopStart = 0
        InLoopStep = 1

        MarginPixel = 0

        For I As Integer = OutLoopStart To OutLoopEnd Step OutLoopStep

            For J As Integer = InLoopStart To InLoopEnd Step InLoopStep

                'ピクセル情報取得
                If Mode = 1 Or Mode = 2 Then
                    PixelColor = TargetBitmap.GetPixel(J, I) '上下
                Else
                    PixelColor = TargetBitmap.GetPixel(I, J) '左右
                End If

                '画像ピクセルのカラーコードを16進数文字列に変換
                PixelHexStr = String.Format("{0:x2}{1:x2}{2:x2}", PixelColor.R, PixelColor.G, PixelColor.B)

                '余白と画像ピクセルの色に差が出たら終了
                If MarginHexStr <> PixelHexStr Then

                    CalcMargin = MarginPixel      '外ループの値を返す
                    Exit Function

                End If

            Next

            MarginPixel = MarginPixel + 1

        Next

    End Function

End Class
